#pragma once

#include <eopcc/ply_io.hpp>
#include <eopcc/txt_io.hpp>
#ifdef LAS_SUPPORT
#include <eopcc/las_io.hpp>
#endif

namespace eopcc {


// Loads a PointCloud using a format inferred from its extension
static void Load(const std::string& filename, PointCloud& point_cloud) {
	std::string ext = filename.substr(filename.find_last_of(".") + 1);

	if (ext.compare("las") == 0 || ext.compare("laz") == 0) {
#ifdef LAS_SUPPORT
		LoadFromLAS(filename, point_cloud);
#else
		throw std::runtime_error("LAS support not built");
#endif
	} else if (ext.compare("ply") == 0) {
		LoadFromPLY(filename, point_cloud);
	} else if (ext.compare("txt") == 0 || ext.compare("csv") == 0) {
		LoadFromTXT(filename, point_cloud);
	} else {
		throw std::runtime_error("unrecognized extension");
	}
}

// Writes a PointCloud using the format inferred from the extension
static void Write(const std::string& filename, PointCloud& point_cloud) {
	std::string ext = filename.substr(filename.find_last_of(".") + 1);

	if (ext.compare("las") == 0 || ext.compare("laz") == 0) {
#ifdef LAS_SUPPORT
		WriteToLAS(filename, point_cloud);
#else
		throw std::runtime_error("LAS support not built");
#endif
	} else if (ext.compare("ply") == 0) {
		WriteToPLY(filename, point_cloud, true);
	} else if (ext.compare("txt") == 0 || ext.compare("csv") == 0) {
		WriteToTXT(filename, point_cloud);
	} else {
		throw std::runtime_error("unrecognized extension");
	}
}


// Factory to create a streaming PointCloudReader
inline PointCloudReader* CreateReader(const std::string& filename) {
	std::string ext = filename.substr(filename.find_last_of(".") + 1);

	if (ext.compare("las") == 0 || ext.compare("laz") == 0) {
		#ifdef LAS_SUPPORT
		return new LASReader(filename);
		#else
		throw std::runtime_error("LAS support not built");
		#endif
	} else if (ext.compare("ply") == 0) {
		return new PLYReader(filename);
	} else if (ext.compare("txt") == 0 || ext.compare("csv") == 0) {
		return new TXTReader(filename);
	} else {
		throw std::runtime_error("unrecognized extension");
	}
}

}
